<?php

namespace Litecms\Rating\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Rating\Http\Requests\RatingRequest;
use Litecms\Rating\Interfaces\RatingRepositoryInterface;
use Litecms\Rating\Models\Rating;

/**
 * Resource controller class for rating.
 */
class RatingResourceController extends BaseController
{

    /**
     * Initialize rating resource controller.
     *
     * @param type RatingRepositoryInterface $rating
     *
     * @return null
     */
    public function __construct(RatingRepositoryInterface $rating)
    {
        parent::__construct();
        $this->repository = $rating;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Rating\Repositories\Criteria\RatingResourceCriteria::class);
    }

    /**
     * Display a list of rating.
     *
     * @return Response
     */
    public function index(RatingRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Rating\Repositories\Presenter\RatingPresenter::class)
                ->$function();
        }

        $ratings = $this->repository->paginate();

        return $this->response->title(trans('rating::rating.names'))
            ->view('rating::rating.index', true)
            ->data(compact('ratings'))
            ->output();
    }

    /**
     * Display rating.
     *
     * @param Request $request
     * @param Model   $rating
     *
     * @return Response
     */
    public function show(RatingRequest $request, Rating $rating)
    {

        if ($rating->exists) {
            $view = 'rating::rating.show';
        } else {
            $view = 'rating::rating.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('rating::rating.name'))
            ->data(compact('rating'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new rating.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RatingRequest $request)
    {

        $rating = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('rating::rating.name')) 
            ->view('rating::rating.create', true) 
            ->data(compact('rating'))
            ->output();
    }

    /**
     * Create new rating.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RatingRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $rating                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('rating::rating.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('rating/rating/' . $rating->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/rating/rating'))
                ->redirect();
        }

    }

    /**
     * Show rating for editing.
     *
     * @param Request $request
     * @param Model   $rating
     *
     * @return Response
     */
    public function edit(RatingRequest $request, Rating $rating)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('rating::rating.name'))
            ->view('rating::rating.edit', true)
            ->data(compact('rating'))
            ->output();
    }

    /**
     * Update the rating.
     *
     * @param Request $request
     * @param Model   $rating
     *
     * @return Response
     */
    public function update(RatingRequest $request, Rating $rating)
    {
        try {
            $attributes = $request->all();

            $rating->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('rating::rating.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('rating/rating/' . $rating->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('rating/rating/' . $rating->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the rating.
     *
     * @param Model   $rating
     *
     * @return Response
     */
    public function destroy(RatingRequest $request, Rating $rating)
    {
        try {

            $rating->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('rating::rating.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('rating/rating/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('rating/rating/' . $rating->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple rating.
     *
     * @param Model   $rating
     *
     * @return Response
     */
    public function delete(RatingRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('rating::rating.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('rating/rating'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/rating/rating'))
                ->redirect();
        }

    }

    /**
     * Restore deleted ratings.
     *
     * @param Model   $rating
     *
     * @return Response
     */
    public function restore(RatingRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('rating::rating.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/rating/rating'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/rating/rating/'))
                ->redirect();
        }

    }

}
